﻿using ItemDragDropSample.Data;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;


// The Grouped Items Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234231

namespace ItemDragDropSample
{
    /// <summary>
    /// A page that displays a grouped collection of items.
    /// </summary>
    public sealed partial class GroupedItemsPage : ItemDragDropSample.Common.LayoutAwarePage
    {
        public GroupedItemsPage()
        {
            this.InitializeComponent();
        }

        protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
            var sampleDataGroups = SampleDataSource.GetGroups((String)navigationParameter);
            this.DefaultViewModel["Groups"] = sampleDataGroups;

            itemGridView.DragItemsStarting += itemGridView_DragItemsStarting;
            itemGridView.DragOver += itemGridView_DragOver;
            itemGridView.Drop += itemGridView_Drop;
        }

        object dragOverItem = null;

        /// <summary>
        /// 드롭 처리
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void itemGridView_Drop(object sender, DragEventArgs e)
        {
            object gridSource;
            e.Data.Properties.TryGetValue("gridSource", out gridSource);
            if (gridSource != sender)
                return;

            object sourceItem;
            e.Data.Properties.TryGetValue("item", out sourceItem);
            if (sourceItem == null)
                return;

            if (dragOverItem == null) return;

            var groups = this.DefaultViewModel["Groups"] as ObservableCollection<SampleDataGroup>;
            var sourceGroupItem = (from kkk in groups
                                   from jjj in kkk.Items
                                   let sIndex = kkk.Items.IndexOf(jjj)
                                   where jjj.UniqueId == (sourceItem as SampleDataCommon).UniqueId
                                   select new { Sgroup = kkk, Sitem = jjj, SItemIndex = sIndex} ).FirstOrDefault();
            var targetGroupItem = (from kkk in groups
                                   from jjj in kkk.Items
                                   let tIndex = kkk.Items.IndexOf(jjj)
                                   where jjj.UniqueId == (dragOverItem as SampleDataCommon).UniqueId
                                   select new { Tgroup = kkk, Titem = jjj, TItemIndex = tIndex }).FirstOrDefault();
            if (sourceGroupItem.Sgroup.Equals(targetGroupItem.Tgroup))
            {
                sourceGroupItem.Sgroup.Items.Move(sourceGroupItem.SItemIndex, targetGroupItem.TItemIndex);
            }
            else
            {
                sourceGroupItem.Sgroup.Items.Remove(sourceGroupItem.Sitem);
                targetGroupItem.Tgroup.Items.Insert(targetGroupItem.TItemIndex, sourceGroupItem.Sitem);
            }
        }

        /// <summary>
        /// 아이뎀이 어떤 위치에있는지 확인
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void itemGridView_DragOver(object sender, DragEventArgs e)
        {
            FrameworkElement element = e.OriginalSource as FrameworkElement;
            var groups = this.DefaultViewModel["Groups"] as ObservableCollection<SampleDataGroup>;
            if (element != null )
            {
                var item = element.DataContext as SampleDataCommon;
                if(item != null)
                {
                    dragOverItem = groups.SelectMany(p => p.Items).FirstOrDefault(p => p.UniqueId == item.UniqueId);
                }
            }
        }

        /// <summary>
        /// 아이템 드래그 시작
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void itemGridView_DragItemsStarting(object sender, DragItemsStartingEventArgs e)
        {
            var item = e.Items.FirstOrDefault();
            if (item == null)
                return;

            e.Data.Properties.Add("item", item);
            e.Data.Properties.Add("gridSource", sender);
        }


        void Header_Click(object sender, RoutedEventArgs e)
        {
            var group = (sender as FrameworkElement).DataContext;

            this.Frame.Navigate(typeof(GroupDetailPage), ((SampleDataGroup)group).UniqueId);
        }

        void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            var itemId = ((SampleDataItem)e.ClickedItem).UniqueId;
            this.Frame.Navigate(typeof(ItemDetailPage), itemId);
        }
    }
}
